//===============================================================================
// PJM Interconnection, LLC. Sprego Application
// Comments provided by Glenn Long
//===============================================================================

using Microsoft.Practices.Composite.Events;

namespace UIComposition.Views.CasesView
{
    using System;
    using System.Linq;
    using System.Windows.Controls;
    using UIComposition.Controllers;
    using UIComposition.Services;
    using System.Windows.Threading;
    using System.Windows.Forms;
    using System.Reflection;
    using System.Configuration;
    using System.Windows.Input;
    using UIComposition.Views.CasesListView;
    using ASEngine.Utilities;
    using System.Windows;
    using System.Timers;
    using System.ComponentModel;
    using System.Threading;
    using UIComposition.PresentationModels;
    using System.Runtime.InteropServices;
    using System.Windows.Interop;
    using System.ServiceModel;
    using PJMBaseLibrary.Utilities.Logger;
    using PJMBaseLibrary.Utilities.ApplicationEnums;
    using System.Windows.Media;
    


    /// <summary>
    /// Interaction logic for CasesListView.xaml
    /// </summary>
    public partial class CasesListView : System.Windows.Controls.UserControl, ICasesListView
    {
        #region Private Members
        private const string CASE_ALERT_RECEIVED = "Case refreshed";
        private const string CASE_CREATED = "Case has been created.";
        private const string CASE_APPROVED = "Case has been approved.";
        private const string CASE_SOLVED = "Case has been solved.";
        private static bool is_running = false;
        public delegate void RefreshUiHandler();
        public delegate void ButtonPressedHandler();
        public delegate void CaseAlertHandler(string msg);
        public delegate void ClientAlertHandler(string msg);

        //public delegate void SelectedNotificationEventHandler(object sender, SelectionChangedEventArgs e);
        //public static event SelectedNotificationEventHandler SelectionModified;

        public delegate void CaseViewUpdateHandler(CasesListPresentationModel casesList);
        private System.Media.SoundPlayer sound = new System.Media.SoundPlayer();
        private TimeData timeData = new TimeData();
        private AlertTimeData alertTimeData = new AlertTimeData();
        private System.Timers.Timer timer = new System.Timers.Timer();
        private System.Timers.Timer alertTimer = new System.Timers.Timer();
        private static bool loaded = false;
        private static int loadingComplete = 0;
        private int findCurrentIndex = 0;
        private static int foundCurrentIndex = 0;
        private static bool alreadySelected = false;
        private static decimal selectedCase = 0;
        

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int SendMessage(IntPtr hWnd, [MarshalAs(UnmanagedType.U4)] int iMsg, int iWParam, int iLParam);
        [DllImport("coredll.dll")]
        public static extern IntPtr SendMessage(IntPtr hWnd, uint uMsg, IntPtr wParam, IntPtr iLParam);

 


        const int WM_HSCROLL = 0x0114;
        const int WM_VSCROLL = 0x0115;

               
        private void Run_Clock()
        {            
            timer.Interval = 1000;                  
            timer.Start();

            alertTimer.Interval = 120000;
            alertTimer.Start();
        }

        #endregion

        #region Public Members
        public CasesListView()
        {
            InitializeComponent();
            Run_Clock();

            if (!Is_Running)
            {               
                // Subscribe to clock ticks
                timer.Elapsed += new ElapsedEventHandler(timer_Elapsed);
                alertTimer.Elapsed += new ElapsedEventHandler(alertTimer_Elapsed);
                //subscribe to the clientcache update event fired from CasesService when receiving DB Alert notification.
                ConnectionService.ClientCacheUpdated += new ConnectionService.ClientCacheUpdatedEventHandler(ClientCacheUpdated);
                 // Show Registered User and Server Time Alert
                ClientController.ClientMessageAlert += new AlertEventHandler(ClientMessageAlert);
                Is_Running = true;
                SaveController.ClientMessageDisplay += new SaveController.MessageEventHandler(ClientMessageDisplay); 
                // Add event to call Tier2 Selection Changed
                //this.Tier1List.SelectionChanged += new SelectionChangedEventHandler(T1CasesList_SelectionChanged);
                //this.Tier2List.SelectionChanged += new SelectionChangedEventHandler(T2CasesList_SelectionChanged);

                 
            }
        }

        
        protected void OnSourceInitialized(EventArgs e)
        {
            base.OnInitialized(e);
            HwndSource source = PresentationSource.FromVisual(this) as HwndSource;
            source.AddHook(WndProc);
        }

        IntPtr WndProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            //if (msg.Msg == WM_HSCROLL && HScrollEvent != null)
            //    HScrollEvent(this, new ScrollEventArgs(ScrollEventType.ThumbTrack, (int)msg.WParam));

            //base.WndProc(ref msg);
            // Handle messages...  
            return IntPtr.Zero;
        } 


        public void LoadButtonPressed()
        {                      
            CasesListView_Loaded();
        }
        
        public void CasesListView_Loaded()
        {
            
            if (this.Tier2List.Items.Count > 0)
            {
                foreach (ASEngine.BusinessObjects.Entities.ASCaseControl item in this.Tier2List.Items)
                {
                    //while (findCurrentIndex < 4)
                    //{
                        if (item.Tier2state.ToString() != "Null" && loaded == false)
                        {
                            // Save record found
                            CasesListView.foundCurrentIndex = findCurrentIndex;

                            Dispatcher.CurrentDispatcher.BeginInvoke(DispatcherPriority.Loaded, (DispatcherOperationCallback)delegate(object arg)
                            {
                                // Choose the first solved or approved item (latest) on the T2 list
                                this.Tier2List.SelectedItem = item;

                                if (CasesListView.foundCurrentIndex == 0)
                                {
                                    ListBoxItem i = (ListBoxItem)this.Tier2List.ItemContainerGenerator.ContainerFromIndex(CasesListView.foundCurrentIndex); // List index is 0 so subtract
                                    if (i != null)
                                    {
                                        i.IsSelected = true;
                                        i.Focus();
                                        selectedCase = item.CASESELECTED;
                                        //i.Background = Brushes.Orange;
                                    }

                                    CasesListView.foundCurrentIndex = 0;
                                }
                                else // Use the saved found value which represents the first solved or approved T2 case
                                {
                                    ListBoxItem i = (ListBoxItem)this.Tier2List.ItemContainerGenerator.ContainerFromIndex(CasesListView.foundCurrentIndex);
                                    if (i != null)
                                    {
                                        i.IsSelected = true;
                                        i.Focus();
                                        selectedCase = item.CASESELECTED;
                                        //i.Background = Brushes.Orange;
                                    }

                                    CasesListView.foundCurrentIndex = 1;
                                }


                                return null;
                            }, null);

                            // We do not need to re-call the CaseSelected method as this action will call it automatically.
                            ASEngine.BusinessObjects.Entities.ASCaseControl selected = item as ASEngine.BusinessObjects.Entities.ASCaseControl;
                            selected.CASESELECTED = Convert.ToDecimal(selected.Tier2caseid);


                            loaded = true;

                        }

                        // Increment to set index of current T2 item
                        findCurrentIndex++;

                    //}
                }

            }
           
        }


        public CasesListPresentationModel Model
        {
            get { return this.DataContext as CasesListPresentationModel; }
            set { CaseUpdater(value); }
        }

        public bool Is_Running
        {
            get { return is_running; }
            set { is_running = value; }
        }

        void timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            timeData.Time = timeData.Time.AddMilliseconds(timer.Interval);
            
            this.Dispatcher.BeginInvoke(
              DispatcherPriority.Loaded,
              (DispatcherOperationCallback)delegate(object arg)
              { this.txtTimeBox.Text = timeData.Time.ToString(); return null; }, null);
           
           
        }

        void alertTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            alertTimeData.Time = alertTimeData.Time.AddMinutes(alertTimer.Interval);

            this.Dispatcher.BeginInvoke(
              DispatcherPriority.Loaded,
              (DispatcherOperationCallback)delegate(object arg)
              { SubscribeToTimeAlert(timeData.Time); return null; }, null);


        }

      

        public static readonly DependencyProperty TimeProperty = DependencyProperty.Register("Time", typeof(DateTime), typeof(TimeData), new PropertyMetadata(DateTime.Now));

        public DateTime Time
        {
            get
            {
                return (DateTime)Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Background,
                    new System.Windows.Threading.DispatcherOperationCallback(delegate { return GetValue(TimeProperty); }), null);
            }
            set
            {
                Dispatcher.BeginInvoke((Action)(() =>
                {
                    SetValue(TimeProperty, value);
                }));
            }
        }

       
        public event EventHandler<DataEventArgs<ASEngine.BusinessObjects.Entities.ASCaseControl>> CaseSelected = delegate { };

        #endregion

        void Tier1List_MouseDown(object sender, MouseButtonEventArgs e)
        {

            if (e.ClickCount == 1)
            {

                ASEngine.BusinessObjects.Entities.ASCaseControl selected = this.Tier1List.SelectedItem as ASEngine.BusinessObjects.Entities.ASCaseControl;

                if (selected.CASESELECTED != selectedCase || selected.CASESELECTED == 0 || selectedCase == 0)
                {
                   // ListBoxItem i = (ListBoxItem)this.Tier1List.ItemContainerGenerator.ContainerFromIndex(CasesListView.foundCurrentIndex); // List index is 0 so subtract

                    if (selected != null)
                    {
                        // Set selected caseID
                        selected.CASESELECTED = Convert.ToDecimal(selected.Tier1caseid);
                        this.caseBlockLable.Text = "Case: " + selected.CASESELECTED.ToString() + " Selected";
                        CaseSelected(this, new DataEventArgs<ASEngine.BusinessObjects.Entities.ASCaseControl>(selected));
                        selectedCase = 0;
                    }
                }
            }
        }

        void Tier2List_MouseDown(object sender, MouseButtonEventArgs e)
        {


            if (e.ClickCount == 1)
            {

                ASEngine.BusinessObjects.Entities.ASCaseControl selected = this.Tier2List.SelectedItem as ASEngine.BusinessObjects.Entities.ASCaseControl;

                if (selected.CASESELECTED != selectedCase || selected.CASESELECTED == 0 || selectedCase == 0)
                {

                    //ListBoxItem i = (ListBoxItem)this.Tier2List.ItemContainerGenerator.ContainerFromIndex(CasesListView.foundCurrentIndex); // List index is 0 so subtract

                    if (selected != null)
                    {
                        // Set selected caseID                   
                        selected.CASESELECTED = System.Convert.ToDecimal(selected.Tier2caseid);
                        this.caseBlockLable.Text = "Case: " + selected.CASESELECTED.ToString() + " Selected";
                        CaseSelected(this, new DataEventArgs<ASEngine.BusinessObjects.Entities.ASCaseControl>(selected));
                        selectedCase = 0;
                    }
                }
            }
            
        }                

        private void T1CasesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
           
            if (e.AddedItems.Count > 0)
            {
                ASEngine.BusinessObjects.Entities.ASCaseControl selected = e.AddedItems[0] as ASEngine.BusinessObjects.Entities.ASCaseControl;

                //ListBoxItem i = (ListBoxItem)this.Tier1List.ItemContainerGenerator.ContainerFromIndex(CasesListView.foundCurrentIndex); // List index is 0 so subtract
 
                if (selected != null)
                {
                    // Set selected caseID
                    selected.CASESELECTED = Convert.ToDecimal(selected.Tier1caseid);
                    this.caseBlockLable.Text = "Case: " + selected.CASESELECTED.ToString() + " Selected";
                    CaseSelected(this, new DataEventArgs<ASEngine.BusinessObjects.Entities.ASCaseControl>(selected));                    
                    selectedCase = selected.CASESELECTED;                    
                }
            } 
        }
        
        private void T2CasesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {

                ASEngine.BusinessObjects.Entities.ASCaseControl selected = e.AddedItems[0] as ASEngine.BusinessObjects.Entities.ASCaseControl;

                //ListBoxItem i = (ListBoxItem)this.Tier2List.ItemContainerGenerator.ContainerFromIndex(CasesListView.foundCurrentIndex); // List index is 0 so subtract
        
                if (selected != null)
                {
                    // Set selected caseID                   
                    selected.CASESELECTED = System.Convert.ToDecimal(selected.Tier2caseid);
                    this.caseBlockLable.Text = "Case: " + selected.CASESELECTED.ToString() + " Selected";
                    CaseSelected(this, new DataEventArgs<ASEngine.BusinessObjects.Entities.ASCaseControl>(selected));
                    selectedCase = selected.CASESELECTED;
                }
            }
       
        }
        

        private void btnAlarmClick(object sender, System.Windows.RoutedEventArgs e)
        {
            this.chkApprovalAlarm.IsChecked = false;
            this.chkSolutionAlarm.IsChecked = false;         

        }       

       

        private void ClientMessageDisplay(string message)
        {
            alarmAreas.Dispatcher.BeginInvoke(DispatcherPriority.Send, new CaseAlertHandler(SaveReminderMessage), message);

        }
        /// <summary>
        /// ClientCacheUpdated - This event is fired from CasesService class to notify the UI of the case refresh.
        /// </summary>
        /// <param name="e"></param>
        private void ClientCacheUpdated(ClientCacheListenerEventArgs e)
        {
            string message = null;
            if (e.ListenerName == "CaseCreatedListener")
            {
                message = e.DCO.CaseID.ToString() + " " + CASE_CREATED + " " + DateTime.Now.ToString();
                alarmAreas.Dispatcher.BeginInvoke(DispatcherPriority.Send, new CaseAlertHandler(CaseAlertDisplay), message);

            }
            if (e.ListenerName == "CaseApprovedListener")
            {
                message = e.DCO.CaseID.ToString() + " " + CASE_APPROVED + " " + DateTime.Now.ToString();
                alarmAreas.Dispatcher.BeginInvoke(DispatcherPriority.Send, new CaseAlertHandler(CaseAlertDisplay), message);

            }
            if (e.ListenerName == "CaseSolvedListener")
            {
                message = e.DCO.CaseID.ToString() + " " + CASE_SOLVED + " " + DateTime.Now.ToString();
                alarmAreas.Dispatcher.BeginInvoke(DispatcherPriority.Send, new CaseAlertHandler(CaseAlertDisplay), message);

            }
        }                     
       
        public void CaseAlertDisplay(string message)
        {
            // Future: Create option to select error type
            txtMessage.Text = message;          
           
          
            //if alarmbox is checked and it is a case approval alert then get sound.
            if ((this.chkApprovalAlarm.IsChecked == true) && (message.Contains(CASE_APPROVED)))
            {
                sound.SoundLocation = ConfigurationSettings.AppSettings["CaseApproveSoundDir"].ToString();

            }
            //if alarmbox is checked and it is a case solved alert then get sound.
            if ((this.chkSolutionAlarm.IsChecked == true) && (message.Contains(CASE_SOLVED)))
            {
                sound.SoundLocation = ConfigurationSettings.AppSettings["CaseSolvedSoundDir"].ToString();

            }
            if (sound.SoundLocation != null)
            {
                sound.LoadAsync();
                sound.Play();
            }
            alarmAreas.Display(txtMessage.Text, "Other");

        }
        //display save changes reminder.
        public void SaveReminderMessage(string message)
        {
            txtMessage.Text = message;
            alarmAreas.Display(txtMessage.Text, "Error");
        }
        public void CaseViewDisplay(CasesListPresentationModel casesList)
        {
            // Future: Create option to select error type

            this.Tier1List.DataContext = casesList.cases;
            this.Tier2List.DataContext = casesList.cases;

            this.Tier1List.ItemsSource = null;
            this.Tier2List.ItemsSource = null;

            this.Tier1List.ItemsSource = (from c in casesList.cases orderby c.MKTHour descending select c).ToList();
            this.Tier2List.ItemsSource = (from c in casesList.cases orderby c.MKTHour descending select c).ToList();
            ug1.DataContext = RunTimesPresentationModel.Instance;   
                 
                
        }

      
        public void ClientMessageAlert(ClientAlertEventArgs e)
        {
            // Presents Data Loading Alert Message
            alarmAreas.Dispatcher.BeginInvoke(DispatcherPriority.Loaded, new CaseAlertHandler(CaseAlertDisplay), e.MessageDetail);
            loadingComplete++;            
            if (loadingComplete >= 11 && loaded == false)
            {
                // Wait another 7 seconds for everything to finish
                System.Threading.Thread.Sleep(7000);
                // Now call the method to load the latest. To do this we will press the Load button which will ensure
                // we remain in context with the view
                this.loadCaseButton.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new ButtonPressedHandler(LoadButtonPressed));                
                 
            }

        }

        public void CaseUpdater(CasesListPresentationModel casesList)
        {
            Dispatcher.BeginInvoke(DispatcherPriority.Input, new CaseViewUpdateHandler(CaseViewDisplay), casesList);          

        }

        //Save Command button
        private void SaveCase_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = SaveController.Instance.IsUpdated;
        }
        private void SaveCase_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            SaveController.Instance.SendUpdates();
        }

        private void MenuItem_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            Environment.Exit(0);
        }

        
        private void MenuItem_Help_About(object sender, System.Windows.RoutedEventArgs e)
        {
            AssemblyName assemName = Assembly.GetExecutingAssembly().GetName();
            string version = assemName.Version.ToString();

            System.Windows.Forms.MessageBox.Show("The Sprego Application Group" +
                       "\nVersion : " + version,
                       "PJM Interconnection LLC",
                       MessageBoxButtons.OK,
                       MessageBoxIcon.Information);

        }

        private void ExecuteCase_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {            
                e.CanExecute = true;
        }

        private void ExecuteCase_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            string resultMsg = DatabaseService.Instance.ExecuteDBFunction(Convert.ToDecimal(CasesService.CaseID), ASConstants.EXECUTE_STRING);
            System.Windows.Forms.MessageBox.Show(resultMsg);
        }

 
        private void ApproveCase_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {            
                e.CanExecute = true;
        }      

        private void ApproveCase_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            string resultMsg = DatabaseService.Instance.ExecuteDBFunction(Convert.ToDecimal(CasesService.CaseID), ASConstants.APPROVE_STRING);
            System.Windows.Forms.MessageBox.Show(resultMsg);
        }

        private void chkApprovalAlarm_Checked(object sender, RoutedEventArgs e)
        {

        }

        private void chkApprovalAlarm_Unchecked(object sender, RoutedEventArgs e)
        {

        }


        private void chkT1AutoExecute_Click(object sender, RoutedEventArgs e)
        {
            string message = "This will turn ";
            string caption = "Warning";
            DialogResult result;
            MessageBoxButtons buttons = MessageBoxButtons.YesNo;

            if (this.chkT1AutoExecute.IsChecked == true)
            {
                message = message + " ON Auto Execute for TIER1 cases. Do you want to continue?";
                result = System.Windows.Forms.MessageBox.Show(message, caption, buttons);

                if (result == System.Windows.Forms.DialogResult.Yes)
                {
                    string resultMsg = DatabaseService.Instance.ExecuteDBFunction(0, ASConstants.AUTOEXECUTET1ON_STRING);
                    if (!resultMsg.Contains("successfully"))
                        System.Windows.Forms.MessageBox.Show(resultMsg);
                }
                else
                {
                    this.chkT1AutoExecute.IsChecked = false;
                }
            }
            else //T1 ischecked is false
            {
                message = message + " OFF Auto Execute for TIER1 cases. Do you want to continue?";
                result = System.Windows.Forms.MessageBox.Show(message, caption, buttons);

                if (result == System.Windows.Forms.DialogResult.Yes)
                {
                    string resultMsg = DatabaseService.Instance.ExecuteDBFunction(0, ASConstants.AUTOEXECUTET1OFF_STRING);
                    if (!resultMsg.Contains("successfully"))
                        System.Windows.Forms.MessageBox.Show(resultMsg);
                }
                else
                {
                    this.chkT1AutoExecute.IsChecked = true;
                }
            }
        }



        private void chkT2AutoExecute_Click(object sender, RoutedEventArgs e)
        {
            string message = "This will turn ";
            string caption = "Warning";
            DialogResult result;
            MessageBoxButtons buttons = MessageBoxButtons.YesNo;
            if (this.chkT2AutoExecute.IsChecked == true)
            {
                message = message + " ON Auto Execute for TIER2 cases. Do you want to continue?";
                result = System.Windows.Forms.MessageBox.Show(message, caption, buttons);

                if (result == System.Windows.Forms.DialogResult.Yes)
                {
                    string resultMsg = DatabaseService.Instance.ExecuteDBFunction(0, ASConstants.AUTOEXECUTET2ON_STRING);
                    if (!resultMsg.Contains("successfully"))
                        System.Windows.Forms.MessageBox.Show(resultMsg);
                }
                else
                {
                    this.chkT2AutoExecute.IsChecked = false;
                }
            }
            else //T1 ischecked is false
            {
                message = message + " OFF Auto Execute for TIER1 cases. Do you want to continue?";
                result = System.Windows.Forms.MessageBox.Show(message, caption, buttons);

                if (result == System.Windows.Forms.DialogResult.Yes)
                {
                    string resultMsg = DatabaseService.Instance.ExecuteDBFunction(0, ASConstants.AUTOEXECUTET2OFF_STRING);
                    if (!resultMsg.Contains("successfully"))
                        System.Windows.Forms.MessageBox.Show(resultMsg);
                }
                else
                {
                    this.chkT2AutoExecute.IsChecked = true;
                }
            }
        }        



        private void chkSolutionAlarm_Checked(object sender, RoutedEventArgs e)
        {

        }

        private void chkSolutionAlarm_Unchecked(object sender, RoutedEventArgs e)
        {

        }
        

        protected bool SubscribeToTimeAlert(DateTime alertTimeData)
        {
            ConnectionService factory = ConnectionService.Instance;
            Core.RemoteControl.IASClient pub = null;

            DuplexChannelFactory<Core.RemoteControl.IASClient> cPublisher = factory.getDuplexChannel();

            if (cPublisher.State == CommunicationState.Opened)
            {
                pub = cPublisher.CreateChannel();

            }

            if (cPublisher == null)
            {
                return false;
            }

            try
            {
                pub.RegisterAlarm(alertTimeData.AddMinutes(1), "Client", "Server Connected");
                return true;
            }
            catch (Exception ex)
            {
                // Write Client Exception
                Logger.LogException(LogSourceType.ClientApplication, ex);
            }

            // Default return value
            return false;


        }

        private void Tier1List_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {            
            ScrollViewer scrollViewer = null;
           
                if (scrollViewer == null)
                {
                    scrollViewer = GetVisualChild<ScrollViewer>(this.Tier2List);
                    scrollViewer.ScrollToVerticalOffset(e.VerticalOffset);
                    Console.Write(scrollViewer.ComputedVerticalScrollBarVisibility);
                }                
            
        }

        private void Tier2List_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            ScrollViewer scrollViewer = null;

            if (scrollViewer == null)
            {
                scrollViewer = GetVisualChild<ScrollViewer>(this.Tier1List);
                scrollViewer.ScrollToVerticalOffset(e.VerticalOffset);
                Console.Write(scrollViewer.ComputedVerticalScrollBarVisibility);
            } 
        }

        public static T GetVisualChild<T>(Visual referenceVisual) where T : Visual
        {
            Visual child = null;
            for (Int32 i = 0; i < VisualTreeHelper.GetChildrenCount(referenceVisual); i++)
            {
                child = VisualTreeHelper.GetChild(referenceVisual, i) as Visual;
                if (child != null && (child.GetType() == typeof(T)))
                {
                    break;
                }
                else if (child != null)
                {
                    child = GetVisualChild<T>(child);
                    if (child != null && (child.GetType() == typeof(T)))
                    {
                        break;
                    }
                }
            }
            return child as T;
        }

       

              
                      
        
    }     


    public class TimeData : DependencyObject
    {
        #region Fields

        DateTime time = DateTime.Now;
        
       
        public DateTime Time
        {
            get { return time; }
            set            
            {
                time = value;
                Dispatcher.CurrentDispatcher.BeginInvoke(DispatcherPriority.DataBind,
                    (SendOrPostCallback)delegate { OnPropertyChanged("Time"); },
                    null);
            }
        }



        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        void OnPropertyChanged(string prop)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(prop));
            }
        }

        #endregion // INotifyPropertyChanged Members

        #endregion
    }

    public class AlertTimeData : DependencyObject
    {
        #region Fields

        DateTime time = DateTime.Now;


        public DateTime Time
        {
            get { return time; }
            set
            {
                time = value;
                Dispatcher.CurrentDispatcher.BeginInvoke(DispatcherPriority.DataBind,
                    (SendOrPostCallback)delegate { OnPropertyChanged("Time"); },
                    null);
            }
        }



        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        void OnPropertyChanged(string prop)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(prop));
            }
        }

        #endregion // INotifyPropertyChanged Members

        #endregion
    }

    public class ListViewClickable : System.Windows.Controls.ListView
    {
        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);
            e.Handled = false;
           
        }

        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonUp(e);
            e.Handled = false;
        }
    }



   }
